/*
 * Jednoduche kresleni v Jave.
 * 
 * Jiri Skala, brezen 2008
 * 
 */
package simpleDrawing;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Jednoduche kresleni v Jave.
 * 
 * @author Jiri Skala
 */
public class SimpleDrawing extends JFrame {

	/**
	 * Panely pro usporadani ovladacich prvku v okne.
	 */
	private JPanel contentPane, panelTools;
	
	/**
	 * Panel kam budeme kreslit.
	 */
	private PanelCanvas panelCanvas;
	
	/**
	 * Prepinaci policka pro tahani/mazani krizku.
	 */
	private JRadioButton radioDrag, radioDelete;
	
	/**
	 * Prepinaci policka pro kresleni celeho obrazku nebo jen vybarveneho tvaru.
	 */
	private JRadioButton radioWholeImage, radioShapeImage;
	
	/**
	 * Skupina prepinacich policek.
	 * Nutne pro spravnou funkci (vzdy jen jedno zaskrtnute).
	 */
	private ButtonGroup radioGroupDrag, radioGroupImage;
	
	/**
	 * Napis.
	 */
	private JLabel labelNote;
	
	/**
	 * Tlacitka.
	 */
	private JButton buttonCrosses, buttonLoad, buttonDraw, buttonSave;
	
	
	
	
	
	/**
	 * Zobrazi okno aplikace.
	 * 
	 * @param args Command line arguments.
	 */
	public static void main(String[] args) {
		new SimpleDrawing().setVisible(true);
	}

	/**
	 * Konstruktor hlavniho okna.
	 */
	public SimpleDrawing() {
		
		// nastavime vlastnosti okna
		this.setTitle("Trochu pokroil kreslen");
		this.setSize(800, 600);
		this.setLocationRelativeTo(null);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		// poskladame GUI
		contentPane = new JPanel(new BorderLayout());
		this.setContentPane(contentPane);
		
		panelTools = new JPanel();
		panelTools.setLayout(new BoxLayout(panelTools, BoxLayout.PAGE_AXIS));
		panelTools.setPreferredSize(new Dimension(200, 200));
		
		RadioChangeListener rchl = new RadioChangeListener();
		radioDrag = new JRadioButton("Tahat kky", true);
		radioDelete = new JRadioButton("Mazat kky");
		radioDelete.addChangeListener(rchl);
		
		radioGroupDrag = new ButtonGroup();
		radioGroupDrag.add(radioDrag);
		radioGroupDrag.add(radioDelete);
		
		labelNote = new JLabel("<html>Pi vytven/mazn muste klikat pesn "
				+ "(nepohybovat pi tom my). Pokud vm to vad, "
				+ "pesute obsluhu udlosti z mouseClicked do mouseReleased.</html>");
		
		ButtonActionListener bal = new ButtonActionListener();
		buttonCrosses = new JButton("Vytvo kky");
		buttonCrosses.addActionListener(bal);
		
		radioWholeImage = new JRadioButton("Kreslit cel obrzek", true);
		radioShapeImage = new JRadioButton("Kreslit vyplnn tvar");
		radioShapeImage.addChangeListener(rchl);
		
		radioGroupImage = new ButtonGroup();
		radioGroupImage.add(radioWholeImage);
		radioGroupImage.add(radioShapeImage);
		
		buttonLoad = new JButton("Otevi obrzek...");
		buttonLoad.addActionListener(bal);
		
		buttonDraw = new JButton("Nakresli nco do obrzku");
		buttonDraw.addActionListener(bal);
		
		buttonSave = new JButton("Ulo obrzek...");
		buttonSave.addActionListener(bal);
		
		panelTools.add(radioDrag);
		panelTools.add(radioDelete);
		panelTools.add(labelNote);
		panelTools.add(buttonCrosses);
		panelTools.add(radioWholeImage);
		panelTools.add(radioShapeImage);
		panelTools.add(buttonLoad);
		panelTools.add(buttonDraw);
		panelTools.add(buttonSave);
		
		panelCanvas = new PanelCanvas();
		
		contentPane.add(panelTools, BorderLayout.WEST);
		contentPane.add(panelCanvas, BorderLayout.CENTER);
	}
	
	
	
	
	
	/**
	 * Trida pro obsluhu udalosti prepinaciho policka.
	 * 
	 * @author Jiri Skala
	 */
	private class RadioChangeListener implements ChangeListener {

		/**
		 * Reakce na zmenu stavu policka.
		 */
		public void stateChanged(ChangeEvent e) {
			
			// ziskame referenci na zdroj udalosti
			Object source = e.getSource();
			
			// zjistime ktere tlacitko bylo stisknuto
			if (source == radioDelete) {
				// nastavime prislusnou operaci
				panelCanvas.setClickDeletes(radioDelete.isSelected());
			}
			
			else if (source == radioShapeImage) {
				// nastavime zpusob kresleni obrazku
				panelCanvas.setDrawFilledShape(radioShapeImage.isSelected());
			}
		}	
	}
	
	/**
	 * Trida pro obsluhu udalosti tlacitka.
	 * 
	 * @author Jiri Skala
	 */
	private class ButtonActionListener implements ActionListener {
		
		/**
		 * Urcuje zda uz byly krizky vytvoreny.
		 */
		private boolean crossesCreated = false;
		
		/**
		 * Oteviraci/ukladaci dialog.
		 */		
		private JFileChooser fileChooser = new JFileChooser();
		
		/**
		 * Reakce na stisk tlacitka.
		 */
		public void actionPerformed(ActionEvent e) {
			
			// ziskame referenci na zdroj udalosti
			Object source = e.getSource();
			
			// zjistime ktere tlacitko bylo stisknuto
			if (source == buttonCrosses) {
				// vytvorime krizky
				if ( ! crossesCreated) {
					panelCanvas.createCrosses(20, 20);
					panelCanvas.repaint();
					crossesCreated = true;
				}
			}
			
			else if (source == buttonLoad) {
				// nacteme obrazek
				int result = fileChooser.showOpenDialog(SimpleDrawing.this);
				if (result == JFileChooser.APPROVE_OPTION)
					panelCanvas.loadImage(fileChooser.getSelectedFile());
			}
			
			else if (source == buttonDraw) {
				// nakreslime neco do obrazku
				panelCanvas.drawToImage();
			}
			
			else if (source == buttonSave) {
				// ulozime obrazek
				int result = fileChooser.showSaveDialog(SimpleDrawing.this);
				if (result == JFileChooser.APPROVE_OPTION)
					panelCanvas.saveImage(fileChooser.getSelectedFile());
			}
		}
	}
}
